@@ -3,8 +3,9 @@ utils::globalVariables(c("l_parses","parses",
                          "l_pos_nums","pos.nums",
                          "l_w_nums","w.nums",
                          "doc_id","sentence_id",
-                         "token_id","token", "."))
-
+                         "token_id","token", ".",
+                         "head_token","head_token_id",
+                         "dep_rel", "tag"))
 
 ################################################################
 # Workflow for SpaCy
@@ -19,45 +20,36 @@ utils::globalVariables(c("l_parses","parses",
 #' @return list of compiled POS-tagged items.
 #' @keywords internal
 #' @import data.table
-spacyParser<-function(txt, num_mc_cores=parallel::detectCores()){
+spacyParser<- function(txt, num_mc_cores=parallel::detectCores()){
   parsedtxt <- spacyr::spacy_parse(txt, dependency=TRUE,lemma=FALSE,pos=TRUE,tag=TRUE,entity=TRUE)
-  parsedtxt$pos.nums<-paste0("(",parsedtxt$token_id,"-",parsedtxt$token,"-",parsedtxt$tag,")")
-  parsedtxt$head_token<-parallel::mclapply(1:nrow(parsedtxt),headTokenGrab, data=parsedtxt, mc.cores=num_mc_cores)
-  parsedtxt[parsedtxt$dep_rel=="ROOT",c("dep_rel","head_token","head_token_id")]<-c("root","ROOT",0)
-  parsedtxt$pos.nums<-paste0("(",parsedtxt$token_id,"-",parsedtxt$token,"-",parsedtxt$tag,")")
-  parsedtxt$parses<-paste0(parsedtxt$dep_rel, "(",parsedtxt$head_token,"-",parsedtxt$head_token_id,", ",parsedtxt$token,"-",parsedtxt$token_id,")")
-  parsedtxt$w.nums<-paste0(parsedtxt$token,"-",parsedtxt$token_id)
+  dt_parsedtxt <- data.table(parsedtxt)
+  unique_doc_ids <- dt_parsedtxt[ , unique(doc_id)]
+  dt_parsedtxt[ , doc_id := factor(doc_id, levels = unique_doc_ids)]
+  dt_head_token <- dt_parsedtxt[ , .(doc_id, sentence_id, token_id,token)]
+  setnames(dt_head_token, c("token_id","token"), c("head_token_id","head_token"))
+  v_s_keys <-  c("doc_id", "sentence_id", "head_token_id" )
+  setkeyv(dt_head_token, v_s_keys)
+  setkeyv(dt_parsedtxt, v_s_keys)
+  dt_parsedtxt <- dt_head_token[dt_parsedtxt] # left merge on dt_parsedtxt
+  dt_parsedtxt[dep_rel=="ROOT" , head_token := "ROOT" ]
+  dt_parsedtxt[dep_rel=="ROOT" , head_token_id := 0 ]
+  dt_parsedtxt[dep_rel=="ROOT" , dep_rel := "root" ]
+  dt_parsedtxt[ , pos.nums := paste0("(",token_id,"-",token,"-",tag,")")]
+  dt_parsedtxt[ , parses := paste0(dep_rel, "(",head_token,"-",head_token_id,", ",token,"-",token_id,")")]
+  dt_parsedtxt[ , w.nums := paste0(token,"-",token_id)]
 
-  dt_parsedtxt <- data.table::data.table(parsedtxt)
-  all.parses <- dt_parsedtxt[ , .(l_parses = list(parses)), by = "doc_id"][ , l_parses]
-  all.pos.nums <- dt_parsedtxt[ , .(l_pos_nums = list(pos.nums)), by = "doc_id"][ , l_pos_nums]
+  all.parses <- dt_parsedtxt[ , .(l_parses = list(parses)), keyby = "doc_id"][ , l_parses]
+  all.pos.nums <- dt_parsedtxt[ , .(l_pos_nums = list(pos.nums)), keyby = "doc_id"][ , l_pos_nums]
   nonums=parallel::mclapply(all.parses,gsub, pattern="-[0-9][0-9][0-9]",replacement="", mc.cores=num_mc_cores)
   nonums=parallel::mclapply(nonums,gsub, pattern="-[0-9][0-9]",replacement="", mc.cores=num_mc_cores)
   nonums=parallel::mclapply(nonums,gsub, pattern="-[0-9]",replacement="", mc.cores=num_mc_cores)
-  w.nums <- dt_parsedtxt[ , .(l_w_nums = list(w.nums)), by = "doc_id"][ , l_w_nums]
+  w.nums <- dt_parsedtxt[ , .(l_w_nums = list(w.nums)), keyby = "doc_id"][ , l_w_nums]
   return(list(parses=all.parses,
               pos.nums=all.pos.nums,
               nonums=nonums,
               w.nums=w.nums))
 }
 
-#' Head Token Grab
-#' @description Compile dependency relation for each row of a SpaCy parse table
-#' @param x row in SpaCy parse table
-#' @param data data.frame SpaCy parse table
-#' @return word tagged as referent from dependencey
-#' @keywords internal
-#'
-headTokenGrab <- function(x, data){
-  data <- data.table(data)
-  doc_id_local <- data$doc_id[x]
-  sentence_id_local <- data$sentence_id[x]
-  head_token_id_local <- data$head_token_id[x]
-  data <- data[(doc_id==doc_id_local)&
-                 (sentence_id==sentence_id_local)&
-                 (token_id==head_token_id_local), ]
-  return(data[,token])
-}
 ################################################################
 # Workflow for coreNLP
 ################################################################
